import typing

from citadel.mqtt import Client

class Light:

    def __init__(self, _id: str, client: Client, **kwargs):
        self._state = None
        self._change_callback = None
        self._available = False

        self._id = _id

        self.STATE_TOPIC = kwargs.get('STATE_TOPIC', 'stat/tasmota/light/{id}/POWER')
        self.STATE_ON_PAYLOAD = kwargs.get('STATE_ON_PAYLOAD', 'ON')
        self.STATE_OFF_PAYLOAD = kwargs.get('STATE_OFF_PAYLOAD', 'OFF')

        self.CMND_TOPIC = kwargs.get('CMND_TOPIC', 'cmnd/tasmota/light/{id}/POWER')
        self.CMND_ON_PAYLOAD = kwargs.get('CMND_ON_PAYLOAD', 'ON')
        self.CMND_OFF_PAYLOAD = kwargs.get('CMND_OFF_PAYLOAD', 'OFF')

        self.LWT_TOPIC = kwargs.get('LWT_TOPIC', 'tele/tasmota/light/{id}/LWT')

        self._client = client
        self._client.subscribe_callback(self.STATE_TOPIC.format(id=self.id), self.onStateChange)
        self._client.subscribe_callback(self.LWT_TOPIC.format(id=self.id), self.onLWTChange)

    def setChangeCallback(self, callback: typing.Callable):
        self._change_callback = callback

    def onLWTChange(self, client, userdata, msg):
        state = msg.payload.decode()
        if state == "Online":
            self._available = True
        elif state == "Offline":
            self._available = False
            self._state = False
        else:
            raise Exception("Unexpected value %s for LWT topic" % state)

        if self._change_callback:
            self._change_callback(self)

    def onStateChange(self, client, userdata, msg):
        state = msg.payload.decode()

        if state == self.STATE_ON_PAYLOAD:
            self._state = True
        elif state == self.STATE_OFF_PAYLOAD:
            self._state = False
        else:
            raise Exception("Unexpected value %s for stat topic" % state)

        if self._change_callback:
            self._change_callback(self)

    def on(self):
        self._client.publish(self.CMND_TOPIC.format(id=self.id), self.CMND_ON_PAYLOAD)

    def off(self):
        self._client.publish(self.CMND_TOPIC.format(id=self.id), self.CMND_OFF_PAYLOAD)

    @property
    def state(self):
        return self._state

    @property
    def available(self):
        return self._available

    @property
    def id(self):
        return self._id


class LightsGroup:

    def __init__(self, _id: str, client: Client, lights: [Light]):
        """ Create a virtual mqtt device that handle multiple light """
        self._id = _id
        self._lights = lights

        for l in self._lights:
            l.setChangeCallback(self.onLightStateChange)

        self._client = client
        self._client.subscribe_callback('cmnd/devices/light/%s/POWER' % self._id, self.cmndPower)

        self._previous_state = all([l.state for l in self._lights])
        self._previous_available = all([l.available for l in self._lights])

        self._client.add_on_connect_callback(self.on_connect)

    def on_connect(self, *args):
        self._client.publish('stat/devices/light/%s/AVAILABLE' % self.id, 'TRUE' if self._previous_available else 'FALSE')

    def cmndPower(self, client, userdata, msg):
        value = msg.payload.decode()
        if value == 'ON':
            self.on()
        elif value == 'OFF':
            self.off()
        elif value == '':
            self._client.publish('stat/devices/light/%s/POWER' % self.id, 'ON' if self.state else 'OFF' )
        else:
            raise Exception("Unexpected value %s for stat topic" % value)

    def onLightStateChange(self, light):
        available = all([l.available for l in self._lights])
        if available != self._previous_available:
            self._previous_available = available
            self._client.publish('stat/devices/light/%s/AVAILABLE' % self.id, 'TRUE' if available else 'FALSE')

        state = all([l.state for l in self._lights])
        if state != self._previous_state:
            self._client.publish('stat/devices/light/%s/POWER' % self.id, 'ON' if state else 'OFF')
            self._previous_state = state

    def on(self):
        for l in self._lights:
            l.on()

    def off(self):
        for l in self._lights:
            l.off()

    @property
    def state(self):
        return self._previous_state

    @property
    def id(self):
        return self._id


